/******************************************************************************
 * ThemeTest 
 * By Rick Sands (Rick@SandsUSA.com)
 * Use what you need
 *****************************************************************************/
#include <PalmOS.h>
#include "StarterRsc.h"

static RGBColorType sysTable[UILastColorTableEntry];

/***********************************************************************
 * CheckThemeOK
 * This routine checks that a ROM version meets your minimum requirement.
 ***********************************************************************/
static Boolean CheckThemeOK(void)
{
	UInt32 romVersion;
	UInt32 requiredVersion = sysMakeROMVersion(3, 5, 0, sysROMStageRelease,0);
	Boolean color;
	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		return false;
	// Only works for color screens!
	WinScreenMode(winScreenModeGet, NULL, NULL, NULL, &color);
	if (!color)
		return false;
	return true;
}

/***********************************************************************
 *  FUNCTION:	ThemeInit
 *  PARAMETERS: sysTableStorage
 ***********************************************************************/
static void ThemeInit(RGBColorType sysTableStorage[])
{
	UInt16 i;
	// Only works for Palm OS 3.5 and higher!
	if (!CheckThemeOK()) return;
	// And stash the startup table ONLY if you need to restore OS default theme
	for (i=0; i<UILastColorTableEntry; i++)
		UIColorGetTableEntryRGB(i, &sysTableStorage[i]);
}

/***********************************************************************
 *  FUNCTION:	ThemeSetOSDefault
 *  PARAMETERS: sysTableStorage
 ***********************************************************************/
static void ThemeSetOSDefault(RGBColorType *sysTableStorage)
{
	UInt16 I;
	if (!CheckThemeOK()) return;
	for (I=0;I<UILastColorTableEntry;I++, sysTableStorage++)
		UIColorSetTableEntry(I, sysTableStorage);
} 

/***********************************************************************
 *  FUNCTION:	ThemeSet
 *  PARAMETERS: ThemeID
 *              = app resource ID of the tclt resource desired
 *				= 0 if you want to reset to Palm Standard.
 ***********************************************************************/
static void ThemeSet(DmResID ThemeID)
{
	UInt16 I;
	MemHandle tTHM;
	MemPtr tTHMptr;
	RGBColorType *cteP;
	if (!CheckThemeOK()) return;
	// Get the theme resource
	tTHM = DmGetResource('tclt', ThemeID);
	if (tTHM)
	{
		// And move it into the UIColorTable
		tTHMptr = MemHandleLock(tTHM);
		cteP = ColorTableEntries(tTHMptr);
		for (I=0;I<UILastColorTableEntry;I++, cteP++)
			UIColorSetTableEntry(I,cteP);
		MemHandleUnlock(tTHM);	
		DmReleaseResource(tTHM);
	}
} 

/***********************************************************************
 * FUNCTION:    MainFormDoCommand
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormPtr frmP;
	switch (command)
	{
		case MainOptionsAboutStarterApp:
				MenuEraseStatus(0);					// Clear the menu status from the display.
				frmP = FrmInitForm (AboutForm);
				FrmDoDialog (frmP);					// Display the About Box.
				FrmDeleteForm (frmP);
				handled = true;
				break;

	}
	return handled;
}


/***********************************************************************
 * FUNCTION:    MainFormHandleEvent
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;

	switch (eventP->eType) 
	{
		case menuEvent:
				return MainFormDoCommand(eventP->data.menu.itemID);
 
	   	case ctlSelectEvent:  // A control button was pressed and released.
	   			switch(eventP->data.ctlEnter.controlID)
	   			{
		   			case MainAlertButton:
				   			FrmAlert(TestAlert);
				   			break;
				   			
				   	case MainSetButton:
				   		{
				   			FormPtr frmP = FrmGetActiveForm();
				   			ListPtr lstP = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, MainThemeList));
							switch (LstGetSelection(lstP))
							{
								case 0: // Application Default
										ThemeSet(1000);
										break;
										
								case 1: // Palm OS Standard (whatever is current)
										ThemeSetOSDefault(sysTable);
										break;
										
								case 2: // Navy
										ThemeSet(1100); // Resource ID
										break;
										
								case 3: // Powder Puff
										ThemeSet(1200); // Resource ID
										break;
										
							}
							FrmGotoForm(MainForm);
				   		}
				   			
			   	}
			   	handled = true;
			   	break;
			   					
		case frmOpenEvent:
				FrmDrawForm(FrmGetActiveForm());
				handled = true;
				break;
	}
	return handled;
}


/***********************************************************************
 * FUNCTION:    AppHandleEvent
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;
	if (eventP->eType == frmLoadEvent)
	{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);
		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
		{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
		}
		return true;
	} 
	return false;
}

 
/***********************************************************************
 * FUNCTION:    AppEventLoop
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);
	} while (event.eType != appStopEvent);
}

/***********************************************************************
 * FUNCTION:    PilotMain
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch: 

			// Only required if you want to allow resetting to Palm Defaults.
			ThemeInit(sysTable);

			// Set the applications default theme (if any)
			ThemeSet(1000);

			FrmGotoForm(MainForm);
			AppEventLoop();
			FrmCloseAllForms();
			break;
	}
	return errNone;
}
